package cc.fascinated.bat.migrations.changelogs;

import com.mongodb.client.FindIterable;
import io.mongock.api.annotations.ChangeUnit;
import io.mongock.api.annotations.Execution;
import io.mongock.api.annotations.RollbackExecution;
import org.bson.Document;
import org.springframework.data.mongodb.core.MongoTemplate;

/**
 * @author Fascinated (fascinated7)
 */
@ChangeUnit(id = "scoresaber-profile-rename-package-changelog", order = "001", author = "fascinated7")
public class ScoresaberProfileRenamePackageChangelog {
    private final MongoTemplate mongoTemplate;

    public ScoresaberProfileRenamePackageChangelog(MongoTemplate mongoTemplate) {
        this.mongoTemplate = mongoTemplate;
    }

    @Execution
    public void changeSet() {
        FindIterable<Document> guilds = mongoTemplate.getCollection("guilds").find();
        guilds.forEach(guild -> {
            Document profiles = guild.get("profiles", Document.class);
            if (profiles == null) {
                return;
            }

            // NumberOneScoreFeedProfile
            Document numberOneScoreFeedProfile = profiles.get("scoresaber-number-one-score-feed", Document.class);
            if (numberOneScoreFeedProfile == null) {
                return;
            }
            numberOneScoreFeedProfile.put("_class", "cc.fascinated.bat.features.scoresaber.profile.guild.NumberOneScoreFeedProfile");
            profiles.put("scoresaber-number-one-score-feed", numberOneScoreFeedProfile);

            guild.put("profiles", profiles);
            mongoTemplate.getCollection("guilds").replaceOne(new Document("_id", guild.get("_id")), guild);
        });

        guilds.forEach(guild -> {
            Document profiles = guild.get("profiles", Document.class);
            if (profiles == null) {
                return;
            }

            // UserScoreFeedProfile
            Document userScoreFeedProfile = profiles.get("scoresaber-number-one-score-feed", Document.class);
            if (userScoreFeedProfile == null) {
                return;
            }

            userScoreFeedProfile.put("_class", "cc.fascinated.bat.features.scoresaber.profile.guild.UserScoreFeedProfile");
            profiles.put("scoresaber-user-score-feed", userScoreFeedProfile);

            guild.put("profiles", profiles);
            mongoTemplate.getCollection("guilds").replaceOne(new Document("_id", guild.get("_id")), guild);
        });

        FindIterable<Document> users = mongoTemplate.getCollection("users").find();
        users.forEach(guild -> {
            Document profiles = guild.get("profiles", Document.class);
            if (profiles == null) {
                return;
            }

            // ScoreSaberProfile
            Document userScoreFeedProfile = profiles.get("scoresaber", Document.class);
            if (userScoreFeedProfile == null) {
                return;
            }

            userScoreFeedProfile.put("_class", "cc.fascinated.bat.features.scoresaber.profile.user.ScoreSaberProfile");
            profiles.put("scoresaber", userScoreFeedProfile);

            guild.put("profiles", profiles);
            mongoTemplate.getCollection("users").replaceOne(new Document("_id", guild.get("_id")), guild);
        });
    }

    @RollbackExecution
    public void rollback() {
        // DO NOTHING
    }
}